--- cdda2wav/ioctl.c.orig
+++ cdda2wav/ioctl.c
@@ -59,10 +59,14 @@
 #include <cdrecord.h>
 
 #if defined (HAVE_IOCTL_INTERFACE)
-#if	!defined	sun	&& !defined	__sun
+#if  !defined(sun) && !defined(__sun) && !(__FreeBSD_version >= 501112)
 static struct cdrom_read_audio arg;
 #endif
 
+#if (defined(__FreeBSD__) && __FreeBSD_version >= 400014) || defined(__DragonFly__)
+static unsigned sector_size = CD_FRAMESIZE;
+#endif
+
 static int err;
 
 static void EnableCdda_cooked __PR((SCSI *scgp, int fAudioMode, unsigned uSectorsize));
@@ -72,13 +76,23 @@
 	int fAudioMode;
 	unsigned uSectorsize;
 {
+#if	(defined(__FreeBSD__) && __FreeBSD_version >= 400014) || defined(__DragonFly__)
+	if (scgp && scgp->verbose)
+		fprintf(stderr, "EnableCdda_cooked (CDRIOCSETBLOCKSIZE)...\n");
+
+	if (fAudioMode) {
+		if (ioctl(global.cooked_fd, CDRIOCGETBLOCKSIZE, &sector_size) ==-1)
+			sector_size = CD_FRAMESIZE;
+        	ioctl(global.cooked_fd, CDRIOCSETBLOCKSIZE, &uSectorsize);
+	} else
+        	ioctl(global.cooked_fd, CDRIOCSETBLOCKSIZE, &sector_size);
+#elif	defined	CDIOCSETCDDA
 	if (scgp && scgp->verbose) {
 		fprintf(stderr, "EnableCdda_cooked (CDIOCSETCDDA)...\n");
 		if (uSectorsize != CD_FRAMESIZE_RAW)
 			fprintf(stderr, "non audio sector size is ignored.\n");
 	}
 
-#if	defined	CDIOCSETCDDA
 	ioctl(global.cooked_fd, CDIOCSETCDDA, &fAudioMode);
 #else
 	fprintf(stderr, "EnableCdda_cooked (CDIOCSETCDDA) is not available...\n");
@@ -156,7 +170,7 @@
 	    fprintf( stderr, "can't get TocEntry #%d lba (error %d).\n", i+1, err );
 	    exit( MEDIA_ERROR );
 	}
-#ifdef	__FreeBSD__
+#if defined(__FreeBSD__) || defined(__DragonFly__)
 	entry[i].cdte_addr.lba = be32_to_cpu(entry[i].cdte_addr.lba);
 #endif
     }
@@ -168,7 +182,7 @@
 	fprintf( stderr, "can't get TocEntry LEADOUT lba (error %d).\n", err );
 	exit( MEDIA_ERROR );
     }
-#ifdef	__FreeBSD__
+#if defined(__FreeBSD__) || defined(__DragonFly__)
     entry[i].cdte_addr.lba = be32_to_cpu(entry[i].cdte_addr.lba);
 #endif
 
@@ -196,7 +210,11 @@
 {
       /* trash the cache */
 
-#if	defined __FreeBSD__
+#if	defined __FreeBSD__ || defined __DragonFly__
+#if	__FreeBSD_version >= 501112
+      pread(global.cooked_fd, (void *) &p[0], 3*CD_FRAMESIZE_RAW,
+          find_an_off_sector(lSector, SectorBurstVal)*CD_FRAMESIZE_RAW);
+#else
       static struct cdrom_read_audio arg2;
 
       arg2.address.lba = find_an_off_sector(lSector, SectorBurstVal);
@@ -206,6 +224,7 @@
 
       ioctl(global.cooked_fd, CDROMREADAUDIO, &arg2);
 #endif
+#endif
 #if	defined __linux__
       static struct cdrom_read_audio arg2;
 
@@ -266,7 +285,18 @@
   static int nothing_read = 1;
 
 /* read 2352 bytes audio data */
-#if	defined __FreeBSD__
+#if	defined __FreeBSD__ || defined __DragonFly__
+#if	__FreeBSD_version >= 501112
+    if (x && x->verbose) {
+	fprintf(stderr, "ReadCdRom_cooked (pread)...\n");
+    }
+
+  do {
+    err = 0;
+    if (pread(global.cooked_fd, (void *) &p[0], SectorBurstVal*CD_FRAMESIZE_RAW,
+	lSector*CD_FRAMESIZE_RAW) == -1)
+		err = -1;
+#else
   arg.address.lba = lSector;
   arg.addr_format = CDROM_LBA;
   arg.nframes = SectorBurstVal;
@@ -279,6 +309,7 @@
   do {
     err = ioctl(global.cooked_fd, CDROMREADAUDIO, &arg);
 #endif
+#endif
 #if	defined __linux__
   arg.addr.lba = lSector;
   arg.addr_format = CDROM_LBA;
@@ -395,7 +426,7 @@
 {
     struct cdrom_subchnl sub_ch;
 
-#if	defined __FreeBSD__
+#if	defined __FreeBSD__ || defined __DragonFly__
     struct cd_sub_channel_info sub_ch_info;
 
     if (x && x->verbose) {
@@ -433,7 +464,7 @@
           return NULL;
       }
       case GET_POSITIONDATA:
-#if	defined __FreeBSD__
+#if	defined __FreeBSD__ || defined __DragonFly__
       sub_ch.data_format = CD_CURRENT_POSITION;
 #endif
 #if defined (__linux__)
